
# q-expansion of newform 3087.2.a.e, downloaded from the LMFDB on 15 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3087
weight = 2
poly_data = [1, -2, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-2, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 1, 0], [0, 0, 0], [2, 0, 0], [0, 0, 0], [0, -1, 0], [-2, 0, -2], [2, -4, 2], [-6, 0, 0], [-5, 3, -5], [-1, -1, 1], [-2, 2, 4], [-8, 1, -4], [-2, 6, -2], [4, -6, 5], [4, 2, -4], [2, -2, 5], [6, 4, 0], [-4, 4, 4], [-5, 1, 1], [-2, -4, 7], [-8, 2, -6], [4, -2, -1], [8, -2, -2], [-4, 8, 2], [4, -6, 10], [2, 6, 4], [2, -8, -2], [-11, 5, -3], [-8, 4, -5], [6, -3, 10], [2, 7, 2], [12, -10, 12], [-5, -5, -7], [4, -12, 2], [-2, 5, -2], [10, -5, 10], [6, -8, -4], [-12, 2, -11], [-6, 2, -16], [-8, -10, 4], [2, 4, -5], [-6, 0, 6], [4, -9, 12], [-21, 1, 3], [5, 9, -5], [10, -16, 14], [-7, 15, 3], [-2, 14, -12], [-8, 12, -14], [-14, 16, -18], [-18, 10, -5], [-3, -11, 5], [-10, 6, -10], [6, -8, 0], [10, -14, -4], [0, -15, 4], [0, -6, -2], [-8, 14, -16], [-8, -1, -8], [-22, 3, 2], [10, 0, 14], [8, 0, 6], [-16, 12, -6], [4, -4, 18], [4, 16, -4], [6, -15, -2], [3, 13, -15], [-12, 1, -4], [0, 0, 12], [14, -6, 8], [-4, -2, 4], [-4, 20, -16], [8, -10, 6], [19, -7, 23], [-16, 0, -8], [-24, 10, -14], [-26, 10, -9], [0, -6, 14], [2, -14, 3], [-2, 2, -12], [18, 2, 6], [-8, 8, 5], [10, 8, 5], [8, -8, -14], [-2, -16, -8], [16, -12, -12], [-14, 14, -19], [-13, 13, -21], [8, 0, -14], [-6, -1, -10], [0, 16, -8], [6, -2, -12], [-20, 16, -4], [6, 16, -15], [-14, -5, 10], [12, -20, 16], [-10, 0, -10], [12, 0, 2], [24, -8, 6], [7, -3, -9], [6, 25, -6], [14, 2, 17], [4, 4, -6], [27, -17, 21], [-20, 14, -3], [-10, 10, -18], [28, -6, 14], [-16, 4, -20], [20, -24, 16], [18, -16, 16], [6, 10, 8], [28, -11, 20], [14, 2, -9], [-12, 0, 2], [13, -17, -9], [-17, 15, 13], [-20, 26, -26], [-22, 14, -18], [-10, -21, 10], [27, 3, 3], [34, -6, 16], [0, -8, 3], [2, -8, -8], [0, 7, 20], [-22, 6, -12], [-22, 15, -2], [4, -24, 3], [18, -12, 30], [-6, 4, -10], [16, -24, 6], [11, -7, 1], [-8, 36, -16], [-22, -15, 6], [9, 15, -11], [-4, 0, -2], [34, -2, 2], [-6, -6, 14], [0, 28, -8], [-42, 8, -2], [22, 1, -18], [2, 14, 8], [2, -6, -15], [8, -10, 7], [0, 4, -16], [0, -10, 14], [6, -12, 14], [4, 6, 4], [-30, 14, -4], [-10, -20, 0], [22, -4, 5], [21, -29, 29], [6, 12, -24], [6, -5, 10], [-26, -16, 14], [-2, 1, -6], [12, 3, 16], [8, -4, 0], [-4, 10, 12], [12, -14, -2], [-2, -10, 14], [18, -4, -1], [14, -11, 2], [1, -1, -13], [8, -18, -26], [-42, 19, -14], [-2, 6, -4], [19, -15, 33], [16, -34, 20], [-1, -7, -25], [-16, 8, -14], [24, -20, 0], [-14, 2, 14], [-25, 19, -37], [8, -28, 23], [0, -30, 6], [-20, 6, 10], [-5, -19, 13], [-1, 3, 9], [42, 6, 4], [18, -10, 0], [-20, 26, -33], [-2, 28, -20], [-8, -12, -5], [-28, 2, 2], [25, -15, 17], [18, -2, -6], [-28, 20, -27], [0, -10, 20], [0, -3, 24], [-20, 30, -4], [34, -22, 38], [-34, -12, 1], [20, 6, 11], [30, -24, 42], [-20, -3, 16], [-40, 10, -28], [-23, 27, -35], [-11, 47, -23], [26, 18, -8], [40, -22, 36], [-11, 9, -17], [-16, 36, -18], [-16, 4, -10], [20, 30, -8], [0, -20, 10], [-22, 20, 0], [2, -2, 2], [15, -25, -1], [-6, -42, 21], [-28, 6, -10], [-4, 37, -8], [-2, -14, 6], [-24, 14, 7], [24, 14, 0], [-26, 30, -14], [6, -2, 16], [-30, 29, -18], [6, -14, -28], [32, -4, 36], [-46, 8, -24], [-22, -32, 20], [-4, -18, 32], [-19, 29, 3], [28, -6, -3], [10, -16, 10], [-4, -3, -16], [-24, 14, -12], [20, 7, -8], [6, -2, 8], [17, 17, 1], [-14, 32, -40], [-14, -2, 28], [-21, 1, 1], [-7, -39, 15], [-16, -2, -8], [16, -12, 16], [-6, 0, 10], [6, -16, 28], [16, -12, 24], [-10, -2, -28], [-1, -9, 11], [10, 16, 16], [-4, -22, 20], [20, -1, 20], [12, 6, 34], [-4, -26, 14], [24, -20, 32], [34, -38, 18], [34, -21, 14], [-46, 20, -11], [-30, 20, 0], [48, -26, 4], [-20, 9, -28], [-36, 2, -14], [-20, 8, -42], [-12, 17, -12], [-8, -11, 28], [-42, 30, -30], [2, 36, -10], [36, -16, 36], [17, -17, -17], [10, 12, 7], [-2, -12, -8], [-18, 54, -20], [10, -6, 38], [-28, -10, 22], [9, -19, 11], [-14, -22, -2], [-29, 9, -23], [-6, 16, 4], [-4, -26, -4], [41, -33, 39], [36, -4, 28], [-33, 21, 5], [10, -8, 34], [-24, 18, -36], [-4, 40, -16], [-17, -15, -5], [0, -49, 32], [-10, -8, -42], [-6, 28, 24], [-26, 5, -10], [4, 8, -2], [30, 4, 34], [32, -24, -16], [2, -4, -15], [-8, -4, 9], [49, -15, 3], [-12, -4, -10], [56, -6, -8], [42, 0, 12], [-26, 6, 8], [10, 23, -2], [-16, 36, -28], [-4, -52, 9], [-12, 8, 26], [-32, 32, -20], [20, 32, -6], [4, -12, 10], [-16, -22, 10], [24, -16, 12], [62, -20, 8], [9, -25, -33], [-16, 5, 20], [-34, 9, -14], [-27, -13, 5], [15, 7, -1], [-14, 50, -34], [2, 0, -23], [44, -49, 40], [-22, 6, 6], [15, -5, -23], [-70, 15, -30], [-18, -13, 14], [28, -25, 48], [-12, 14, 34], [16, -6, -32], [-46, 8, -7], [4, -30, 8], [14, -54, 15], [8, -20, -16], [-32, -12, -13], [38, -20, 10], [2, 9, 18], [-2, 6, 21], [-14, 30, -32], [32, 22, -15], [-46, 26, -12], [13, -15, -27], [-42, 4, -32], [-12, 16, -7], [-26, 34, 3], [42, 6, 32], [60, -12, 2], [30, 3, 10], [22, -8, 50], [28, -39, -32], [72, -30, 50], [6, 6, -24], [16, 26, -36], [-25, -39, 7], [-22, 2, -21], [32, -16, -28], [-8, -22, 11], [-30, -8, -28], [-10, -4, -21], [-12, 6, 14], [24, -14, 11], [12, -64, 12], [10, -20, -18], [-28, 14, 10], [-12, -12, 37], [46, -19, 10], [82, -16, 20], [52, 4, 5], [4, -26, 34], [30, -24, 44], [-14, -6, 10], [-38, -27, 18], [52, 7, -8], [-4, 19, 32], [54, -38, 74], [21, -27, 21], [0, 28, -36], [-22, 44, -26], [-43, -1, -11], [-54, 36, -32], [8, 2, -5], [27, 5, -1], [6, 16, 13], [-8, 0, 18], [52, -35, 12], [54, -15, -6], [-52, 16, 2], [12, -20, -19], [44, -30, 18], [15, 47, -9], [-44, 30, -28], [62, -14, 0], [44, 10, -22], [-46, 4, 14], [21, -23, 3], [20, -4, 28], [-26, 53, -30], [-40, 30, -60], [-72, 30, -60], [20, -42, 27], [10, -22, 24], [22, -31, 58], [-30, 32, -2], [-3, 7, 21], [36, 22, -2], [-13, 3, -61], [-52, 12, 12], [31, 31, -5], [-11, -45, -7], [0, 4, -46], [56, -48, 26], [0, 22, -37], [14, -4, -20], [42, 4, 32], [-18, 1, -6], [28, 18, -9], [12, 6, 24], [8, -38, 2], [24, -70, 44], [3, 61, -9], [-54, 30, -23], [-40, -16, 16], [-6, -34, -6], [-8, -46, -2], [-10, 31, 42], [16, 36, 8], [10, 1, -26], [37, 23, -31], [-34, 66, -32], [16, -29, 56], [24, 22, 2], [-16, 6, -6], [-6, 30, -10], [-33, 75, -47], [12, -40, 42], [-25, 11, -13], [40, 8, -6], [22, -20, -8], [40, 2, 8], [-26, -28, -9], [14, -22, 7], [-36, -38, 16], [-4, -12, -16]]
